# This file is part of the Falcon repository creator
# Copyright (C) 2005-2006 Dennis Kaarsemaker
# See the file named COPYING in the root of the source tree for license details

import falcon.util
import falcon.conf

import os, re

template = None
cache = {}
@falcon.util.timer
def generate(dir, mirror = None):
    # Bail out if dir doesn't exist
    if not os.path.exists(dir):
        falcon.util.warning("Not updating html, %s does not exist." % dir)
        falcon.util.warning("Forgot to run falcon update?")
        return
    # Which file to generate?
    # - main index in / -> basename(dir) == conf/mirror.basedir
    # - main index in dists/$release -> basename(dirname(dir)) == 'dists'
    # - component index in dists/$release/$component -> basename(dir) == component name
    do_main = False
    do_release_main = False
    name = 'component ' + os.path.basename(dir)
    conf = falcon.conf
    if mirror:
        if dir == mirror['basedir']:
            do_main = True
            name = 'repository main'
    else:
        if dir == falcon.conf.basedir:
            do_main = True
            name = 'repository main'
    if os.path.basename(os.path.dirname(dir)) == 'dists':
        do_release_main = True
        name = 'release ' + os.path.basename(dir)
        release = os.path.basename(dir)
        
    if mirror:
        falcon.util.debug("Creating %s index for mirror %s" % (name, mirror['alias']))
        webbase = mirror['webbase']
        sponsor = mirror['sponsor']
        if do_release_main:
            components = falcon.util.intersect(mirror['components'], falcon.conf.components[release])
    else:
        falcon.util.debug("Creating %s index" % name)
        webbase = falcon.conf.webbase
        sponsor = None
        if do_release_main:
            components = falcon.conf.components[release]
    while webbase[-1] == '/':
        webbase = webbase[:-1]
    
    if do_main:
        if mirror and os.path.exists(os.path.join(falcon.conf.basedir, 'desc-%s' % mirror['alias'])):
            df = os.path.join(falcon.conf.basedir, 'desc-%s' % mirror['alias'])
        else:
            df = os.path.join(falcon.conf.basedir, 'desc')
        if df not in cache:
            cache[df] = falcon.util.readfile(df)
        repodesc = cache[df]
        releases = {}
        for r in falcon.conf.releases:
            df = os.path.join(falcon.conf.basedir,'pool',r,'desc')
            if df not in cache:
                cache[df] = falcon.util.readfile(df)
                if not cache[df]:
                    cache[df] = "No description"
            releases[r] = cache[df]
        releasekeys = sorted(releases.keys())
        # Export GPG key
        if falcon.conf.gpgkey and 'falcon-gpgkey' not in cache:
            cache['falcon-gpgkey'] = falcon.util.run('gpg --export --armor %s' % falcon.conf.gpgkey)
        falcon.util.writefile(os.path.join(dir, '%s.gpg' % falcon.conf.gpgkey),cache['falcon-gpgkey'])
    elif do_release_main:
        repodesc = cache[os.path.join(falcon.conf.basedir,'pool',release,'desc')]
        components = {}
        if mirror and not (mirror['components'] == ['all']):
            sects = falcon.util.intersect(mirror['components'],falcon.conf.components[release])
        else:
            sects = falcon.conf.components[release]
        for sect in sects:
            df = os.path.join(falcon.conf.basedir, 'pool', release, sect, 'desc')
            if sect == 'all':
                cache[df] = "This is the 'all' metacomponent containing all packages in all components."
            else:
                if df not in cache:
                    cache[df] = falcon.util.readfile(df)
                    if not cache[df]:
                        cache[df] = 'No description'
            desc = cache[df]

            #if os.path.basename(dir) == falcon.conf.release:
            components[sect] = (sect, desc)
            #else:
            #    # No os.path.join here: these are URLs
            #    components[sect] = ("dists/%s/%s/" % (falcon.conf.release, sect), desc)
        componentkeys = sorted(components.keys())
        sectionkeys = componentkeys
        sections = components
    else:
        release = os.path.basename(os.path.dirname(dir))
        component = os.path.basename(dir)
        section = component
        desc = cache[os.path.join(falcon.conf.basedir, 'pool', release, component, 'desc')]
        sources, sourcekeys = falcon.scanner.scan_sources(os.path.join(falcon.conf.basedir, 'dists', release, component))
        if not len(sourcekeys):
            falcon.util.warning("Component %s does not contain any packages" % component)
            return

    # Now fill the template
    t = template(searchList=[locals()])
    falcon.util.writefile(os.path.join(dir, 'index.html'), str(t))

def format_changelog(cl):
    out = '<dl>'
    cl = cl.strip().replace('&','&amp;').replace('<','&lt;').replace('>','&gt;')
    # Strip 'empty' lines
    cl = re.sub(' (  )*\.\n','',cl)
    # Split into versions
    versions = [x for x in re.split('(?:^|\n) (?=\S)', cl, re.DOTALL | re.M) if x.strip()]
    for v in versions:
        (ver, log) = v.split("\n",1)
        out += '<b>%s</b>' % ver
        # Join the lines of items
        log = re.sub(r"\n (?:  )*([^*\-+ ])",r"\1", log)
        # Magic :)
        for line in log.split("\n"):
            l = (len(line) - len(line.lstrip()) -1) / 2
            out += '<ul><li>' * l
            out += line.lstrip()[1:]
            out += '</li></ul>' * l
    return out
